package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.Diet
import com.aitrainer.api.model.diet.DietMeal
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietTest {

    @Autowired
    private lateinit var mockMvc: MockMvc


    private val diet = Diet(
        dietUserId = 1,
        dietText = "Test diet text",
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert diet successfully`() {

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(diet))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietUserId").value(1))
            .andReturn()

        val gson= Gson()
        val newDietJson = mvcResult.response.contentAsString
        val newDiet = gson.fromJson(newDietJson, Diet::class.java)

        mockMvc.perform(get("/api/diet/{dietUserId}", 1)
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].dietText").value("Test diet text"))

        val meal = DietMeal(
            mealName = "monday|breakfast",
            meal = "Főtt tojás"
        )
        val meal2 = DietMeal(
            mealName = "monday|lunch",
            meal = "Disznópörkölt"
        )
        val meal3 = DietMeal(
            mealName = "monday|lunch",
            meal = "Savanyúkáposzta"
        )

        newDiet.meals.add(meal)
        newDiet.meals.add(meal2)
        newDiet.meals.add(meal3)

        //update
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet/${newDiet.dietId}")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(newDiet))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.dietUserId").value(1))
            .andExpect(jsonPath("$.meals[0].meal").value("Főtt tojás"))
            .andExpect(jsonPath("$.meals[1].mealName").value("monday|lunch"))



    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
